;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This script used to plot e3sm diagnostics for TCs;;;;;;
;TC Basin definitions.(Knutson et. al.,2020,BAMS)
;North Atlantic: 0-90N, 265-360E
;Northeast Pacific: 0-90N, 180-265E
;Northwest Pacific: 0-90N, 100-180E
;North Indian: 0-90N, 30-100E
;South Indian: 90S-0, 20-135E
;Southwest Pacific: 90S-0, 135-295E
;South Atlantic(SA1+SA2): 90S-0, South America (295-360E) to Africa (0-20E)
;The North Atlantic-northeast Pacific boundary is on a
;diagonal tracing a path through Mexico and Central America.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  sgrd      = "2x2"
  figkey    = "ivt_ssp585_" + sgrd
  datdir    = "/anvil/projects/x-phy220062/x-szhang3/evaluation/ar_statistics/ar_info"+"_" + sgrd
  Figstr    = (/"(a)", "(b)", "(a)", "(b)", "(c)", "(d)"/)
  ystr      = 1979
  yend      = 2014
  nyr       = yend - ystr +1
  cbstr     = "Track density (storms/year/"+sgrd+" degree box)"

  Varnam    = "ivt_ar"
  Varstr    = "AR IVT" 
  lbtitle   = "~F33~D~N~~F~IVT (kg m~S~-1~N~ s~S~-1~N~)"

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Groups    = (/"CLIM30", "CLIM30ML","SSP585", "SSP585ML"/)
  LabList   = (/"CLIM", "ML(CLIM)","SSP585 - CLIM", "ML(SSP585) - ML(CLIM)"/)
  ngrps     = dimsizes(Groups)
  timtag    = Groups
  timtag    = ystr+"-"+yend
  nyrs      = yend - ystr + 1

  do i = 0,ngrps-1,1

   fna  = datdir+"/"+Groups(i)+"_AR_statistics_"+timtag(i)+"_"+sgrd+".nc"
   print(fna)
   fla  = addfile(fna, "r")
   lat  = fla->lat
   lon  = fla->lon
   time = fla->time 

  ;calculate the ssp585e mean of annual total  
   var0       = reshape(fla->$Varnam$,(/nyrs,12,dimsizes(lat),dimsizes(lon)/))
   vart       = dim_avg_n_Wrap(var0,1) 
   vart!0     = "time"
   vart!1     = "lat"
   vart!2     = "lon"
   vart&lat   = lat 
   vart&lon   = lon 
   vart&time  = ispan(ystr,yend,1)

   print(" ")
   print("........................... ")
   print("igp : " + i) 
   print("........................... ")

   if(Groups(i).eq."CLIM30") then
     vref0 = vart
   end if

   if(Groups(i).eq."CLIM30ML") then
     vref1 = vart
   end if

   if(i.le.1) then 
     var  = dim_avg_n_Wrap(vart,0)
     plot_sig_test = False
   else 
     if(Groups(i).eq."SSP585".or.Groups(i).eq."SSP585") then
       vref  = vref0
     end if
     if(Groups(i).eq."SSP585ML".or.Groups(i).eq."SSP585ML") then
       vref  = vref1
     end if
     var   = dim_avg_n_Wrap(vart,0)
     var   = var - dim_avg_n(vref,0)
     ;;student-t test
     sigr  = 0.05                                 ; critical sig lvl for r
     xtmp  = vref(lat|:,lon|:,time|:)
     ytmp  = vart(lat|:,lon|:,time|:)
     xAve  = dim_avg (xtmp)              ; calculate means at each grid point
     yAve  = dim_avg (ytmp)
     xVar  = dim_variance (xtmp)         ; calculate variances
     yVar  = dim_variance (ytmp)
     xEqv  = equiv_sample_size (xtmp, sigr,0)
     yEqv  = equiv_sample_size (ytmp, sigr,0)

     xN    = dimsizes(vref(:,0,0)) ;wgt_areaave (xEqv, wgty, 1., 0)    ; wgty could be gaussian weights
     yN    = dimsizes(vart(:,0,0)) ;wgt_areaave (yEqv, wgty, 1., 0)
     iflag = False                        ; population variance similar
     prob  = var
     prob  = (/ttest(xAve,xVar,xN,yAve,yVar,yN,iflag, False)/)
     print(xN+ " " + yN)
     printMinMax(prob,False)
     prob  = where(prob.lt.sigr,prob,prob@_FillValue)
     plot_sig_test = True

     delete([/xtmp,ytmp,xAve,yAve,xVar,yVar,sigr,xEqv,yEqv,xN,yN,iflag/])

   end if 
 
   ;calculate the global/regional mean values
   rad   = 4.*atan(1.)/180.    ;
   wgty  = cos(lat*rad)      ; cosine weights
   gmn   = wgt_areaave (var,wgty,1.0,0)
   lstr  =  Figstr(i) + "  " +LabList(i) + " (" +Varstr+")"
   if(i.le.1) then 
     rstr =  "Global mean (%): " + sprintf("%.1f",gmn)
   else
     rstr =  "Mean bias (%): " + sprintf("%.1f",gmn)
   end if 
   delete([/var0,vart,fna,fla,lat,lon,time/])

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------

   if(i.eq.0)then
    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
    wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
 
    plotname = "track_" + figkey 
    wks1     = gsn_open_wks(wtype,plotname)
    plot     = new (ngrps, graphic)

    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")

   end if 

   ;;;set up the plot information;;;;
   lndcolor   = "Black" ;"Grey50"
   if(sgrd.eq."4x4") then 
     cnlevels  := (/0.1,0.2,0.5,1,2,3,4,5,6,7,8/) ;1.5,2.0/)
     cnlevel2  := (/-2.0,-1.0,-0.5,-0.2,-0.1,0.1,0.2,0.5,1.0,2.0/)
   else
     cnlevels  := (/0.1,1,5,10,20,30,40,50,60,70,80,90/)
     cnlevel2  := (/-20.,-15.,-10.,-5.,-1.0,1.0,5.,10.,15.,20./)
   end if 

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 0.5
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.020
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres     = res
   resx      = res
   resx1     = res

   res@gsnStringFontHeightF  = FontHeightF*2.0
   res@lbLabelFontHeightF    = FontHeightF*1.0
   res@lbTitleFontHeightF    = FontHeightF*1.0

   res@cnFillOn              = True     ; color plot desired
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "Transparent";"White"

   res@lbLabelBarOn             = True   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.05
  ;res@lbOrientation            = "Horizonal" ; "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.16 ;0.60
   res@pmLabelBarWidthF         = 0.55 ;0.06
   res@lbTitleString            = ""
   res@lbTitlePosition          = "Top" ;"Right"         ; title location
  ;res@lbTitleDirection         = "Across"        ; letter angle
  ;res@lbTitleAngleF            = 90.          ; title angle
   res@lbBoxLinesOn             = True
  ;res@lbBoxSeparatorLinesOn    = False
  ;res@lbLabelStride            = 15

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

;---Control appearance of map.
;  mpres@gsnMajorLatSpacing     = 30
;  mpres@gsnMajorLonSpacing     = 45
;  mpres@gsnMinorLatSpacing     = 15
;  mpres@gsnMinorLonSpacing     = 15

   ;mpres@mpOutlineBoundarySets = "GeophysicalAndUSStates"
   ;mpres@mpOutlineBoundarySets = "AllBoundaries"
   mpres@mpDataBaseVersion     = "MediumRes"
   mpres@mpDataSetName         = "Earth..2"

   mpres@mpProjection          = "Robinson"
   mpres@mpFillOn              = False
   mpres@mpPerimOn             =  False 
   mpres@mpGridAndLimbOn       =  True
   mpres@mpGridLineThicknessF  = 0.1
   mpres@mpGridLatSpacingF     =  30.0
   mpres@mpGridLonSpacingF     =  30.0
   mpres@mpGeophysicalLineColor      = lndcolor  ; color of cont. outlines
   mpres@mpGeophysicalLineThicknessF = 0.1 ;0.1  
  ;mpres@mpGridAndLimbDrawOrder="PostDraw"
   mpres@mpGridLineColor        = -1;"transparent" ; trick ncl into drawing earth's outline
   mpres@mpMinLatF              = -90  
   mpres@mpMaxLatF              =  90 
   mpres@mpCenterLonF           = 180.0

   mpres@tmYLLabelsOn     = True
   mpres@tmYLOn           = True
   mpres@tmYROn           = False

   plot(i) = gsn_csm_map(wks1,mpres)

;---Resources for plotting original data
   res1=res
   res1@gsnAddCyclic                = True
   res1@cnLevelSelectionMode        = "ExplicitLevels"  
  ;res1@cnFillPalette               = "ViBlGrWhYeOrRe";"testcmap" ;"NCV_jaisnd";"temp_19lev"

   res1@cnLinesOn                   = False ;plot_contour  
   res1@cnLineLabelsOn              = False  
   res1@cnInfoLabelOn               = False
   res1@cnLineLabelDensityF         = 2.0
   res1@cnLineLabelFontHeightF      = 0.020
   res1@cnLineLabelBackgroundColor  = -1
   res1@gsnContourLineThicknessesScale = 1.0
   res1@cnLineThicknessF            = 0.001
   res1@cnLineColor                 = "Black"
   res1@gsnContourZeroLineThicknessF= 0.2
   res1@gsnContourNegLineDashPattern= 0

   res1@gsnLeftString                 = lstr
   res1@gsnLeftStringParallelPosF     = 0.05
   res1@gsnLeftStringOrthogonalPosF   = 0.04
   res1@gsnRightString                = "";rstr
   res1@gsnRightStringOrthogonalPosF  = 0.04
   res1@gsnCenterString               = ""
   res1@gsnCenterStringOrthogonalPosF = 0.04

   if(i.le.1) then 
     res1@cnFillPalette               = "wh-bl-gr-ye-re";"precip2_17lev"
     res1@cnLevels                   := cnlevels
     res1@lbTitleString               = lbtitle;
   else
     res1@cnFillPalette               = "CBR_coldhot";"precip2_17lev"
     res1@cnLevels                   := cnlevel2
     res1@lbTitleString               = lbtitle;
   end if 

   plot_org1  = gsn_csm_contour(wks1,var,res1)
   overlay(plot(i) ,plot_org1)

   if(plot_sig_test) then
     res21 = res
     res21@gsnLeftString                = ""
     res21@gsnRightString               = ""
     res21@gsnCenterString              = ""
     res21@cnMissingValFillColor        = "Transparent"
     res21@cnFillOn                     = True
     res21@cnLinesOn                    = False
     res21@cnLineLabelsOn               = False       ; turn off contour labels
     res21@cnInfoLabelOn                = False
     res21@cnMonoFillColor              = True        ; default color is fground [black]
     res21@cnMonoFillPattern            = True
     res21@cnMonoFillScale              = True
     res21@cnFillPattern                = 17
     res21@cnFillScaleF                 = 0.9
     res21@cnFillDotSizeF               = 0.001
     res21@lbLabelBarOn                 = False
     plot_cont2 = gsn_csm_contour(wks1,prob,res21)
     overlay(plot(i),plot_cont2)
     delete([/res21/])
   end if

 delete([/res,res1,mpres,var/])

end do ; end of experiment group loop

;---Draw both plots in a panel
;***************************************
; panel first set of plots
;***************************************
  pres1                          = True
  pres1@gsnFrame                 = True      ; don't advance frame yet
  pres1@gsnMaximize              = False
  pres1@gsnPanelLabelBar         = False
  pres1@pmLabelBarParallelPosF   = 0.0
  pres1@pmLabelBarOrthogonalPosF = -0.02
 ;pres1@lbOrientation            = "vertical"  ; vertical label bar
  pres1@pmLabelBarHeightF        = 0.05 ;0.21
  pres1@pmLabelBarWidthF         = 0.75 ;0.05
  pres1@lbLabelFontHeightF       = FontHeightF * 0.6
  pres1@lbTitleFontHeightF       = FontHeightF * 0.65
  pres1@lbTitleString            = cbstr 
  pres1@lbTitlePosition          = "Top" ;"Right"         ; title location
 ;pres1@lbTitleDirection         = "Across"        ; letter angle
 ;pres1@lbTitleAngleF            = 90.          ; title angle
  pres1@lbBoxLinesOn             = True
 ;pres1@lbBoxSeparatorLinesOn    = False
 ;pres1@lbLabelStride            = 15

  pres1@gsnPanelMainString       = " "
  pres1@gsnPanelMainFontColor    = "Black"
  pres1@gsnPanelMainFontHeightF  = 0.028
  gsn_panel(wks1,(/plot(2:)/),(/1,2/),pres1)

end
